/// <reference path="typings/youtube/youtube.d.ts" />
/// <reference path="typings/jquery/jquery.d.ts" />
var TubeBack;
(function (TubeBack) {
    'use strict';
    //#region variables
    TubeBack.elementClass = 'youtube-background';
    TubeBack.videoBackgrounds = [];
    TubeBack.youtubeApiReady = jQuery.Deferred();
    TubeBack.resizeDebounceTime = 20;
    //#endregion variables
    //#region events
    if (!window.hasOwnProperty('yt')) {
        var tag = document.createElement('script');
        tag.src = 'https://www.youtube.com/iframe_api';
        var firstScriptTag = document.getElementsByTagName('script')[0];
        firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);
    }
    window.onYouTubeIframeAPIReady = function () {
        TubeBack.youtubeApiReady.resolve();
    };
    var resizeVideos = debounce(function () {
        for (var i = 0; i < TubeBack.videoBackgrounds.length; i++) {
            if (TubeBack.videoBackgrounds[i].intiated) {
                TubeBack.videoBackgrounds[i].resizeVideo();
            }
        }
    }, TubeBack.resizeDebounceTime);
    jQuery(window).on('resize', resizeVideos);
    function onPlayerStateChange(event) {
        var player = event.target;
        var state = event.data;
        var videoBackground = getVideoBackgroundById(jQuery(player.f).attr('id'));
        switch (state) {
            case YT.PlayerState.PLAYING:
                if (typeof videoBackground.options.onPlayerStart === 'function') {
                    videoBackground.options.onPlayerStart(event);
                }
                break;
            case YT.PlayerState.ENDED:
                if (typeof videoBackground.options.onPlayerEnded === 'function') {
                    videoBackground.options.onPlayerEnded(event);
                }
                break;
            case YT.PlayerState.PAUSED:
                if (typeof videoBackground.options.onPlayerPaused === 'function') {
                    videoBackground.options.onPlayerPaused(event);
                }
                break;
            case YT.PlayerState.BUFFERING:
                if (typeof videoBackground.options.onPlayerBuffering === 'function') {
                    videoBackground.options.onPlayerBuffering(event);
                }
                break;
        }
    }
    function onPlayerReady(event) {
        var player = event.target;
        player.mute();
        var videoBackground = getVideoBackgroundById(jQuery(player.f).attr('id'));
        if (player.getDuration <= videoBackground.options.startFrom) {
            videoBackground.options.startFrom = 0;
        }
        if (player.getDuration() > 10) {
            var timer = setInterval(function () {
                videoBackground.playWhenBuffered(timer);
            }, 150);
        }
        else {
            videoBackground.play();
        }
    }
    //#endregion interfaces    
    function getVideoBackgroundById(id) {
        console.time('getVideoBackgroundById');
        for (var i = 0, length = TubeBack.videoBackgrounds.length; i < length; i++) {
            if (TubeBack.videoBackgrounds[i].uniqueId === id) {
                console.timeEnd('getVideoBackgroundById');
                return TubeBack.videoBackgrounds[i];
            }
        }
        return null;
    }
    function debounce(func, wait, immediate) {
        var timeout;
        return function () {
            var context = this, args = arguments;
            var later = function () {
                timeout = null;
                if (!immediate)
                    func.apply(context, args);
            };
            var callNow = immediate && !timeout;
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
            if (callNow)
                func.apply(context, args);
        };
    }
    ;
    function createGUID() {
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
                .toString(16)
                .substring(1);
        }
        return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
            s4() + '-' + s4() + s4() + s4();
    }
    function getSize(height, width, ratio) {
        if (width / ratio < height) {
            width = Math.ceil(height * ratio);
        }
        else {
            height = Math.ceil(width * ratio);
        }
        return { height: height, width: width };
    }
    var VideoBackground = (function () {
        function VideoBackground(element, options) {
            var _this = this;
            this.intiated = false;
            this.element = jQuery(element);
            this.uniqueId = createGUID();
            this.options = this.setDefaults(options);
            var size = getSize(this.element.parent().innerHeight(), this.element.parent().innerWidth(), this.options.ratio);
            this.options.height = size.height;
            this.options.width = size.width;
            if (!this.element.find('.' + TubeBack.elementClass + '-container').length) {
                if (this.element.css('position') === 'static') {
                    this.element.css({
                        'position': 'relative'
                    });
                }
                if (this.element.css('z-index') === 'auto') {
                    this.element.css('z-index', '1');
                }
                this.element.css({
                    'background-image': 'url(' + this.options.fallbackImage + ')',
                    'background-size': 'cover',
                    'background-position': 'center',
                    'background-repeat': 'no-repeat',
                });
                var containerElement = jQuery('<div></div>', {
                    'class': TubeBack.elementClass + '-container'
                }).css({
                    'background-image': 'url(' + this.options.fallbackImage + ')',
                    'background-position': 'center center',
                    'background-size': 'cover',
                    'background-repeat': 'no-repeat',
                    'padding': '0',
                    'margin': '0',
                    'position': this.element.css('display') === 'flex' ? 'absolute' : 'relative',
                    'overflow': 'hidden',
                    'flex': 'none',
                    'z-index': '-1'
                });
                var videoElement = jQuery('<div></div>', {
                    'class': TubeBack.elementClass,
                    'id': this.uniqueId
                }).css({
                    'pointer-events': 'none',
                    'position': 'absolute',
                    'top': '50%',
                    'left': '50%',
                    'transform': 'translate(-50%,-50%)',
                    'opacity': '0',
                    'transition': 'opacity 0.5s ease'
                });
                containerElement.append(videoElement);
                this.element.prepend(containerElement);
                TubeBack.videoBackgrounds.push(this);
                if (window.hasOwnProperty('yt')) {
                    this.initialize();
                }
                else {
                    TubeBack.youtubeApiReady.then(function () {
                        _this.initialize();
                    });
                }
            }
        }
        VideoBackground.prototype.initialize = function () {
            if (!this.intiated) {
                var player = new YT.Player(this.uniqueId, {
                    height: this.options.height,
                    width: this.options.width,
                    videoId: this.options.videoId,
                    playerVars: {
                        autoplay: 1,
                        controls: 0,
                        autohide: 1,
                        loop: this.options.loop ? 1 : 0,
                        cc_load_policy: 0,
                        modestbranding: 1,
                        iv_load_policy: 3,
                        showinfo: 0,
                        playlist: this.options.loop ? [this.options.videoId] : null,
                        rel: 0
                    },
                    events: {
                        onReady: onPlayerReady,
                        onStateChange: onPlayerStateChange
                    }
                });
                this.player = player;
                this.intiated = true;
            }
        };
        VideoBackground.prototype.setDefaults = function (options) {
            var _options = {
                videoId: options.videoId,
                ratio: 16 / 9,
                mute: true,
                loop: true,
                height: 0,
                width: 0,
                startFrom: 0,
                onPlayerStart: options.onPlayerStart,
                onPlayerEnded: options.onPlayerEnded,
                onPlayerPaused: options.onPlayerPaused,
                onPlayerBuffering: options.onPlayerBuffering,
                onPlayerCued: options.onPlayerCued
            };
            if (typeof options !== 'undefined') {
                _options.ratio = typeof options.ratio === 'number' ? options.ratio : _options.ratio;
                _options.mute = typeof options.mute === 'boolean' ? options.mute : _options.mute;
                _options.loop = typeof options.loop === 'boolean' ? options.loop : _options.loop;
                _options.height = typeof options.height === 'number' ? options.height : _options.height;
                _options.width = typeof options.height === 'number' ? options.width : _options.width;
                _options.startFrom = typeof options.startFrom === 'number' ? options.startFrom : _options.startFrom;
                _options.fallbackImage = (options.fallbackImage) ? options.fallbackImage : ('http://img.youtube.com/vi/' + _options.videoId + '/maxresdefault.jpg');
            }
            else {
                _options.fallbackImage = 'http://img.youtube.com/vi/' + _options.videoId + '/maxresdefault.jpg';
            }
            return _options;
        };
        VideoBackground.prototype.resizeVideo = function () {
            var playerElement = jQuery(this.player.f);
            var backgroundContainer = playerElement.parent();
            backgroundContainer.css({
                'height': backgroundContainer.parent().innerHeight() + 'px',
                'width': backgroundContainer.parent().innerWidth() + 'px',
                'margin-left': '-' + backgroundContainer.parent().css('padding-left'),
                'margin-top': '-' + backgroundContainer.parent().css('padding-top'),
                'margin-bottom': '-' + backgroundContainer.parent().innerHeight() + 'px',
            });
            var parentWidth = backgroundContainer.innerWidth();
            var parentHeight = backgroundContainer.innerHeight();
            var size = getSize(parentHeight, parentWidth, this.options.ratio);
            this.options.height = size.height;
            this.options.width = size.height;
            this.player.setSize(size.width, size.height);
        };
        ;
        VideoBackground.prototype.play = function () {
            this.options.mute ? this.player.mute() : this.player.unMute();
            if (this.options.startFrom != 0) {
                if (typeof this.options.startFrom === 'number') {
                    this.player.seekTo(this.options.startFrom, true);
                }
            }
            this.player.playVideo();
            this.resizeVideo();
            jQuery(this.player.f).css('opacity', '1');
            if (this.options.loop) {
                this.loop();
            }
        };
        VideoBackground.prototype.playWhenBuffered = function (timer) {
            var buffered = false;
            if (this.options.startFrom != 0 && (this.options.startFrom + 5) <= this.player.getDuration()) {
                buffered = this.player.getVideoLoadedFraction() >= this.options.startFrom / this.player.getDuration();
            }
            else {
                buffered = this.player.getVideoLoadedFraction() >= 10 / this.player.getDuration();
            }
            if (buffered) {
                window.clearInterval(timer);
                this.play();
            }
        };
        VideoBackground.prototype.loop = function () {
            if (this.player.getDuration() > 10) {
                setInterval(function () {
                    this.player.seekTo(this.options.startFrom, true);
                }, (((this.player.getDuration() * 1000) - 4000) - (this.options.startFrom * 1000)));
            }
        };
        return VideoBackground;
    })();
    TubeBack.VideoBackground = VideoBackground;
})(TubeBack || (TubeBack = {}));
(function ($) {
    $.fn.tubeback = function (options) {
        if (typeof options.videoId !== 'undefined') {
            return new TubeBack.VideoBackground(this, options);
        }
        else {
            console.error('tubeback: No videoId provided');
        }
    };
})(jQuery);
